@@ -46,7 +46,7 @@ sub get {
     }
 
     $state_grid->[0] = \@start_states;
-    for my $row_i (1 .. $size_y - 1) { # compute next rows  say "row $row_i :";
+    for my $row_i (1 .. $size_y - 1) { # compute next row
         ($state_grid->[$row_i], $action_grid->[$row_i]) 
             = $iterator->( $state_grid->[$row_i - 1], $action_grid->[$row_i - 1]);
     }    
@@ -68,11 +68,14 @@ sub compile_iterator {
     my $act_size = 3;
     my $states = $config->{'global'}{'state_count'};
     my $rule_count = $states ** $rule_size;
+    my $action_count = $states ** $act_size;
     my $x_skew_factor = int $rule_size / 2;
     my $a_skew_factor = 1;
+    my $last_x_index = $comp_size_x - 1;
     # code head
-    my $code = 'sub { my ($state_row, $action_row) = @_;'."\n";  # $code .= 'say "l ", int @$state_row;';
-    $code .= 'my ($new_srow, $new_arow) = ([],[(0) x $comp_size_x]); my $state = 0; my $active = 0;'."\n";
+    my $code = 'sub { my ($state_row, $action_row) = @_;'."\n";
+    $code .= 'my ($new_state_row, $new_action_row, $arow) = ([], [(0) x $comp_size_x], [(0) x $comp_size_x]);'."\n";
+    $code .= 'my $state = 0; my $active = 0;'."\n";
     # row start #say "tf -> ", int @$transfer_function, "  take avg ",$config->{'rules'}{'avg'};
     my $shift_val = '$state *= '.$states.';';
     my $crop_val = '$state %= '.$rule_count.';';
@@ -92,8 +95,8 @@ sub compile_iterator {
     } else {
         $code .= $shift_val.'$state += $state_row->['.$_.'];'."\n" for reverse 1 .. $x_skew_factor; 
     }
-    $code .= '$new_srow->[0] = $transfer_function->[$state];'."\n";
-    $code .= '$new_arow->[0] = $action_function->[$active];'."\n";
+    $code .= '$new_state_row->[0] = $transfer_function->[$state];'."\n";
+    $code .= '$arow->[0] = $action_function->[$state];'."\n";
     for my $cell_i( 1 .. $x_skew_factor - 1){
         $code .= $config->{'rules'}{'avg'} 
               ? '$state -= $state_row->['.($comp_size_x - $x_skew_factor + $cell_i).'];'."\n"
@@ -104,28 +107,28 @@ sub compile_iterator {
                    : '$state += ( $state_row->['.($cell_i - 1).'] * $plus_factor) - ($state_row->['.$cell_i.'] * $minus_factor);'."\n";
         }
         $code .= '$state += $state_row->['.($cell_i + $x_skew_factor).'];';
-        $code .= '$new_srow->['.$cell_i.'] = $transfer_function->[$state];'; 
-        $code .= '$new_arow->['.$cell_i.'] = $action_function->[$active];';
+        $code .= '$new_state_row->['.$cell_i.'] = $transfer_function->[$state];'; 
+        $code .= '$arow->['.$cell_i.'] = $action_function->[$state];'; 
     }
     my $plus_factor = $states ** ($x_skew_factor);
     my $minus_factor = $states ** ($x_skew_factor-1);
-    my $row_stop = $comp_size_x - 1 - $x_skew_factor;
+    my $row_stop = $last_x_index - $x_skew_factor;
     # main loop
     $code .= 'for my $cell_i'." ($x_skew_factor .. $row_stop){"."\n"; 
    
     unless ($rule_size % 2){
         $code .= $config->{'rules'}{'avg'}
-               ? '$state += $state_row->[$cell_i - 1] - $state_row->[$cell_i];'."\n"
-               : '$state += ( $state_row->[$cell_i - 1] * $plus_factor) - ($state_row->[$cell_i] * $minus_factor);'."\n";
+               ? '  $state += $state_row->[$cell_i - 1] - $state_row->[$cell_i];'."\n"
+               : '  $state += ( $state_row->[$cell_i - 1] * $plus_factor) - ($state_row->[$cell_i] * $minus_factor);'."\n";
     }
     $code .= $shift_val unless $config->{'rules'}{'avg'}; 
-    $code .= '$state += $state_row->[$cell_i + '.$x_skew_factor.'];'."\n"; 
+    $code .= '  $state += $state_row->[$cell_i + '.$x_skew_factor.'];'."\n"; 
     $code .= $config->{'rules'}{'avg'} ? '$state -= $state_row->[$cell_i - $x_skew_factor - 1];'."\n" : $crop_val;
-    $code .= '$new_srow->[$cell_i] = $transfer_function->[$state];'."\n"; 
-    $code .= '$new_arow->[$cell_i] = $action_function->[$active];'."\n"; 
+    $code .= '  $new_state_row->[$cell_i] = $transfer_function->[$state];'."\n";
+    $code .= '  $arow->[$cell_i] = $action_function->[$state];'; 
     $code .= '}'."\n";
     # loop tail
-    for my $cell_i( $comp_size_x - $x_skew_factor .. $comp_size_x - 1){
+    for my $cell_i( $comp_size_x - $x_skew_factor .. $last_x_index){
         $code .= $config->{'rules'}{'avg'} 
               ? '$state -= $state_row->['.($cell_i - $x_skew_factor - 1).'];'."\n"
               : $shift_val . $crop_val."\n";
@@ -135,49 +138,33 @@ sub compile_iterator {
                    : '$state += ( $state_row->['.($cell_i - 1).'] * $plus_factor) - ($state_row->['.$cell_i.'] * $minus_factor);'."\n";
         }
         $code .= '$state += $state_row->['.($cell_i - $comp_size_x + $x_skew_factor).'];' if $config->{'global'}{'circular_grid'};
-        $code .= '$new_srow->['.$cell_i.'] = $transfer_function->[$state];'; 
-        $code .= '$new_arow->['.$cell_i.'] = $action_function->[$active];'; 
-    }; # say $code;
+        $code .= '$new_state_row->['.$cell_i.'] = $transfer_function->[$state];'."\n"; 
+        $code .= '$arow->['.$cell_i.'] = $action_function->[$state];'; 
+    }; 
+    # compute action rules
+    $code .= '$new_action_row->['.$last_x_index.'] |= 1 if $arow->[0] & 4;'."\n" if $config->{'global'}{'circular_grid'}; 
+    $code .= '$new_action_row->[0] |= 1 if $arow->[0] & 2;'."\n";
+    $code .= '$new_action_row->[1] |= 1 if $arow->[0] & 1;'."\n";
 
-    eval $code.'($new_srow, $new_arow) }';
+    $code .= 'for my $cell_i ( 1 ..'.($last_x_index - 1)."){\n"; 
+    $code .= '  $active = $arow->[$cell_i];'."\n"; 
+    $code .= '  $new_action_row->[$cell_i - 1 ] |= 1 if $active & 4;'."\n";
+    $code .= '  $new_action_row->[$cell_i     ] |= 1 if $active & 2;'."\n";
+    $code .= '  $new_action_row->[$cell_i + 1 ] |= 1 if $active & 1;'."\n";
+    $code .= "}\n"; 
+    $code .= '$active = $arow->['.$last_x_index.'];'."\n";
+    $code .= '$new_action_row->['.($last_x_index - 1).'] |= 1 if $active & 4;'."\n";
+    $code .= '$new_action_row->['.$last_x_index.']       |= 1 if $active & 2;'."\n";
+    $code .= '$new_action_row->[ 0 ] |= 1                     if $active & 1;'."\n" if $config->{'global'}{'circular_grid'};
+    
+    # apply action rules
+    $code .= 'for ( 0 ..'.$last_x_index."){\n"; 
+    $code .= '    $new_state_row->[$_] = $state_row->[$_] unless $action_row->[$_];'."\n"; 
+    $code .= "}\n".'($new_state_row, $new_action_row) }'; # return solution
+    eval $code; # say $code;
 }
 
 1;
 
-# - activity grid
+# - flexible activity grid
 __END__
-        #~ my $row = $state_grid->[$row_i] = [];
-                #~ my $act = $action_grid->[$row_i]  = [(0) x $comp_size_x];
-                #~ $row->[0]  = $bact->[0] ? $transfer_function->[ $val ] : $brow->[0];
-                #~ $act->[0] |= 1 if $action_function->[ $val ] & 2;
-                #~ $act->[1] |= 1 if $action_function->[ $val ] & 1;            
-                #~ for my $cell_i (1 .. $size_x - 2) {
-                    #~ $row->[$cell_i] = $bact->[$cell_i] ? $transfer_function->[ $val ] : $brow->[$cell_i];
-                    #~ $act->[$cell_i-1] |= 1 if $action_function->[ $val ] & 4;
-                    #~ $act->[$cell_i  ] |= 1 if $action_function->[ $val ] & 2;
-                    #~ $act->[$cell_i+1] |= 1 if $action_function->[ $val ] & 1;            
-                #~ }
-                #~ # for (1 .. $rule_size - 1)
-                    #~ $row->[$cell_i] = $bact->[$cell_i] ? $transfer_function->[ $val ] : $brow->[$cell_i];
-                    #~ $act->[$cell_i-1] |= 1 if $action_function->[ $val ] & 4;
-                    #~ $act->[$cell_i  ] |= 1 if $action_function->[ $val ] & 2;
-
-sub { 
-    my ($state_row, $action_row) = @_;
-    my ($new_srow, $new_arow) = ([],[]); 
-    my $state = 0;
-    $state *= 2;
-    $state += $state_row->[198];
-    $state *= 2;
-    $state += $state_row->[0];
-    $state *= 2;
-    $state += $state_row->[1];
-    $new_srow->[0] = $state;
-    for my $cell_i (1 .. 197){
-        $state *= 2;
-        $state += $state_row->[$cell_i + 1];
-        $state %= 8;
-        $new_srow->[$cell_i] = $state;
-    }
-    ($new_srow, $new_arow) 
-}
